# -*- coding: utf-8 -*-

from wtforms import Form, SelectField, validators as v, StringField, BooleanField, FormField,\
    ValidationError
from fmtapper_web.frontend.forms import TapperUIBaseForm
from form_utils import validate_time_interval, validate_ftp_conf_field, validate_ftp_time_interval, is_not_empty, \
    validate_latin, validate_feed_url, validate_spaces, validate_channel_id
from fmtapper_web.tuner import Tuner, BroadcastConfig


def abs_path(form, field):
    dir_path = field.data
    if not dir_path.startswith('/'):
        raise ValidationError(u'Необходимо указать абсолютный путь.')


class EncodingConfigForm(TapperUIBaseForm):
    bit_rate = StringField(u'Частота дискретизации', [v.number_range(min=0)])
    sample_rate = StringField(u'Битрейт выходного потока', [v.number_range(min=0)])
    mono = BooleanField(u'Моно')
    url = StringField(u'Маска файла', [v.required(), abs_path, validate_latin])
    interval_length = StringField(u'Длина интервала', [v.optional(), v.number_range(min=0, max=86399)])
    fragment_length = StringField(u'Длина фрагмента', [v.optional(), v.number_range(min=1, max=86399)])


class FeedConfigForm(TapperUIBaseForm):
    password = StringField(u'Пароль', [v.optional()])
    content_type = StringField(u'Тип контента', [v.optional()])
    ice_genre = StringField(u'Жанр стрима', [v.optional()])
    ice_name = StringField(u'Имя стрима', [v.optional()])
    ice_description = StringField(u'Описание стрима', [v.optional()])
    is_public = BooleanField(u'Публичный стрим', [v.optional()])
    legacy_icecast = BooleanField(u'IceCast версии менее 2.4', [v.optional()])


class IceCastFeedConfigForm(TapperUIBaseForm):
    password = StringField(u'Пароль', [v.required()])
    content_type = StringField(u'Тип контента', [v.required(), validate_latin])
    ice_genre = StringField(u'Жанр стрима', [v.optional()])
    ice_name = StringField(u'Имя стрима', [v.optional()])
    ice_description = StringField(u'Описание стрима', [v.optional()])
    is_public = BooleanField(u'Публичный стрим', [v.optional()])
    legacy_icecast = BooleanField(u'IceCast версии менее 2.4', [v.optional()])


class BroadcastConfigForm(TapperUIBaseForm):
    server_type = SelectField(u'Сервер вещания', choices=[('0', 0), ('1', 1)], validators=[v.optional()])
    feed_url = StringField(u'URL трансляции', [validate_latin, validate_feed_url])
    stream_url = StringField(u'URL потока', [validate_latin])
    feed_config = FormField(FeedConfigForm)

    def validate(self):
        form_valid = Form.validate(self)
        if not form_valid:
            updated = dict()
            for field in self.errors:
                updated['broadcast_config.' + field] = self.errors[field]
            self._errors = updated
            self.errors.update(updated)

        feed_config_valid = True
        if self.server_type.data == str(BroadcastConfig.BroadscastType.ICECAST) and is_not_empty(self.feed_url.data):
            form = IceCastFeedConfigForm(data=self.feed_config.data, prefix='broadcast_config.feed_config')
            feed_config_valid = form.validate()
            if not feed_config_valid:
                updated = dict()
                for field in form.errors:
                    updated['broadcast_config.feed_config' + '.' + field] = form.errors[field]
                form._errors = updated
                self.errors.update(updated)
        return form_valid and feed_config_valid


class AudioDeviceConfigForm(TapperUIBaseForm):
    name = StringField(u'Имя устройства', [v.required()])
    sample_rate = StringField(u'Частота дискретизации', [v.number_range(min=1)])


class UsbDeviceConfigForm(TapperUIBaseForm):
    name = StringField(u'Имя устройства', [v.required()])
    sample_rate = StringField(u'Частота дискретизации', [v.number_range(min=1)])
    frequency = StringField(u'Частота FM-диапазона', [v.number_range(min=1)])


class FmDeviceConfigForm(TapperUIBaseForm):
    host = StringField(u'Хост', [validate_latin])
    port = StringField(u'Порт', [])
    frequency = StringField(u'Частота FM-диапазона', [v.number_range(min=1)])
    udp_port = StringField(u'UDP-порт', [])
    tuner_sample_rate = StringField(u'Частота дискретизации тюнера', [v.number_range(min=1)])


class InetRadioConfigForm(TapperUIBaseForm):
    source_url = StringField(u'URL потока', [v.url()])


class DeviceConfigForm(TapperUIBaseForm):
    name = StringField(u'Имя устройства')
    sample_rate = StringField(u'Частота дискретизации', [v.optional(), v.number_range(min=1)])
    host = StringField(u'Хост', [v.optional(), v.ip_address()])
    port = StringField(u'Порт', [v.optional(), v.number_range(min=1)])
    udp_port = StringField(u'UDP-порт', [v.optional(), v.number_range(min=1)])
    tuner_sample_rate = StringField(u'Частота дискретизации тюнера', [v.optional(), v.number_range(min=1)])
    frequency = StringField(u'Частота FM-диапазона', [v.optional(), v.number_range(min=1)])
    source_url = StringField(u'URL потока', [v.optional(), v.url()])


class FtpConfigForm(TapperUIBaseForm):
    ftp_url = StringField(u'Адрес удалённого хранилища')
    ftp_user = StringField(u'Имя пользователя', [validate_ftp_conf_field])
    ftp_pwd = StringField(u'Пароль', [validate_ftp_conf_field])
    ftp_metadata = BooleanField(u'Загружать метаданные')
    ftp_upload_interval = StringField(u'Интервал закачки', [validate_ftp_conf_field, validate_ftp_time_interval])
    ftp_status = StringField(u'Статус')


def validate_device_config(device_type, field):
        prefix = 'device_config'
        if device_type == str(Tuner.DeviceType.AUDIO_DEVICE):
            cls = AudioDeviceConfigForm
        elif device_type == str(Tuner.DeviceType.USB_TUNER):
            cls = UsbDeviceConfigForm
        elif device_type == str(Tuner.DeviceType.FM_TUNER):
            cls = FmDeviceConfigForm
        else:
            cls = InetRadioConfigForm

        form = cls(data=field.data, prefix=prefix)
        res = form.validate()
        if not res:
            updated = dict()
            for field in form.errors:
                updated[prefix + '.' + field] = form.errors[field]
            form._errors = updated
        return res, form


class TunerForm(TapperUIBaseForm):
    id = StringField(u'Id тюнера', [v.required(), v.length(min=1, max=20), validate_latin, validate_spaces])
    name = StringField(u'Имя тюнера', [v.required(), v.length(min=1)])
    channel_id = StringField(u'Номер канала', [v.required(), validate_latin, validate_spaces, validate_channel_id])
    device_type = SelectField(u'Тип устройства', choices=[('0', 0), ('1', 1), ('2', 2), ('3', 3)],
                              validators=[v.required()])
    device_config = FormField(DeviceConfigForm)
    encoding_config = FormField(EncodingConfigForm, [v.required()])
    broadcast_config = FormField(BroadcastConfigForm, [v.required()])
    store_audio = StringField(u'Время хранения аудиоматериалов', [v.required(), validate_time_interval])
    store_logs = StringField(u'Время хранения логов', [v.number_range(min=0)])
    schedule = BooleanField(u'Расписание')
    volume_window = StringField(u'Окно громкости', [v.number_range(min=0)])
    ftp_config = FormField(FtpConfigForm)

    def validate(self):
        form_valid = Form.validate(self)

        if 'broadcast_config' in self.errors:
            self.errors.update(self.errors['broadcast_config']);
            self.errors.pop('broadcast_config')

        (device_valid, device_config_form) = validate_device_config(self.device_type.data, self.device_config)
        if not device_valid:
            self.errors.update(device_config_form.errors)
        return form_valid and device_valid
